<?php

namespace catchAdmin\map\controller;

use catcher\base\CatchRequest as Request;
use catcher\CatchResponse;
use catcher\base\CatchController;
use Aliyun\OTS\OTSClient;

use catchAdmin\device\model\Device as deviceModel;
use catchAdmin\fences\model\Fences as fencesModel;
use think\facade\Db;
use think\facade\Env;
use PDO;
class RouteMap extends CatchController
{
    protected $deviceModel;
    protected $fencesModel;
    protected $filterTrack=false;
    public function __construct(deviceModel $deviceModel,fencesModel $fencesModel)
    {
        $this->deviceModel = $deviceModel;
        $this->fencesModel = $fencesModel;
    }
    
    /**
     * 列表
     * @time 2021年05月18日 10:15
     * @param Request $request 
     */
    public function queryDeviceList(Request $request) : \think\Response
    {
        $list =$this->deviceModel->getList();
        foreach($list as &$item){
            if($item['online_time']<$item['wifi_online_time']){
                $item['longitude']=$item['wifi_longitude'];
                $item['latitude']=$item['wifi_latitude'];
            }
        }
        return CatchResponse::paginate($list);
    }
     /**
     * 列表
     * @time 2021年05月18日 10:15
     * @param Request $request 
     */
    public function  queryDeviceRealTimeInfo(Request $request) : \think\Response
    {
        $params=$request->param();
        if(!isset($params['imeis'])){
            return CatchResponse::success([]);
        }
        // var_dump($params);   
        $imeis=$params['imeis'];
        
        $list=$this->deviceModel->whereIn('imei',$imeis)->select();
        foreach ($list as &$value) {
            $value['status_icon']='run';
            $value['status']='在线';
         }
        // var_dump($list);
        return CatchResponse::success($list);
    }
    /**
     * 查询告警列表
     * @time 2021年05月24日
     * @return \think\response\Json
     */
    public function queryAlarmList(Request $request): \think\Response
    {
        $list=$this->deviceModel->queryAlarmList();


        foreach($list as &$item){
            if($item['online_time']<$item['wifi_online_time']){
                $item['longitude']=$item['wifi_longitude'];
                $item['latitude']=$item['wifi_latitude'];
            }
           
        }
        return CatchResponse::paginate($list);
    }
    /**
     *
     * @time  2021年05月24日
     * @return \think\response\Json
     */
    public function queryLocationList(Request $request): \think\Response
    {
        return CatchResponse::success($this->deviceModel->queryLocationList());
    }
    
    /**
     *
     * @time  2021年05月24日
     * @return \think\response\Json
     */
    public function queryAlarmLocationList(Request $request): \think\Response
    {
        return CatchResponse::success($this->deviceModel->queryAlarmLocationList());
    }
    
    
    /**
     *
     * @time  2021年05月24日
     * @return \think\response\Json
     */
    public function queryFenceList(Request $request): \think\Response
    {
        $list=DB::table('fences')->where('fence_type',0)->select()->toArray();
        foreach($list as &$val){
            $user_depart=Db::table('users')->where('id',$val['creator_id'])->value('department_id');
            $creator_level=Db::table('departments')->where('id',$user_depart)->value('level');
            if(!$creator_level){
                $val['level']=1;
            }else{
                $level_arr=explode('-',$creator_level);
                $val['level']=count($level_arr)+1;
            }
        }
        return CatchResponse::success($list);
    }

    /**
     * 查询当前定位
     * @time 2021年05月18日 10:15
     * 
     */
    public function queryDeviceCurLocation(Request $request) : \think\Response
    {
        $params=$request->param();

        $device_number=$params['device_number'];
        if (!$device_number) {
            return CatchResponse::fail("获取设备号失败");
        }
        $device_info=$this->deviceModel->where('imei',$device_number)->find()->toArray();
        // var_dump($device_info);
        if($device_info['online_time']<$device_info['wifi_online_time']){
            $device_info['online_time']=$device_info['wifi_online_time'];
            $device_info['longitude']=$device_info['wifi_longitude'];
            $device_info['latitude']=$device_info['wifi_latitude'];
        }
        return CatchResponse::success($device_info);
    }
    


    /**
     * 查询学生轨迹GPS
     * @time 2021年05月18日 10:15
     * 
     */
    public function queryDeviceRouteGps(Request $request) : \think\Response
    {
        
        $params=$request->param();
        // var_dump($params);
        // var_dump(strtotime($params['aData']));
        $device_number = $params['imei'];

        if (!$device_number) {
            return CatchResponse::fail("获取设备号失败");
        }
        if($params['time']==0 && isset($params['time_range']) && $params['time_range'] != ''){
            $start_time=strtotime($params['time_range'][0]);
            $end_time=strtotime($params['time_range'][1]);
           
        }else{
            $start_time = time()-$params['time'];
            $end_time=time();
        }
        


        $options=array(
            'device_number'=>$device_number,
            'start_time'=>$start_time,
            'end_time'=>$end_time,
            'table_name'=>'wxt_route_gps'
        );
        $res = queryTableStoreGpsRoute($options);
        if($res['success']){
            $data=$res['data'];
            $battery_level=Db::table('devices')->where('imei',$device_number)->value('battery_level');
            foreach($data as &$val){
                $val['Longitude']=round($val['Longitude'],6);
                $val['Latitude']=round($val['Latitude'],6);
                $val['SignalType']='GPS';
                $val['BatteryLevel']=$battery_level;
    
            }
            return CatchResponse::success($data);
        }else{
            return CatchResponse::fail($res['message']);
        }
        
       

        
        
    }

    
    
     /**
     * 查询告警相关围栏信息
     * @time 2021年05月18日 10:15
     * @param Request $request 
     */
    public function queryAlarmFenceList(Request $request) : \think\Response
    {
        $params=$request->param();
        $record=$params['record'];
        $fence_list=array();
        $alarm_list=$record['alarm_reason_list'];
        foreach($alarm_list as $val){
            $fence_id=Db::table('alarm_report')
                ->where('device_number',$record['imei'])
                ->where('alarm_type',$val['value'])
                ->order('id', 'desc')
                ->value('fence_id');

            if($fence_id){
                $fence_info=$this->fencesModel->where('id',$fence_id)->append(['type_name','in_fence_time_area_text','out_fence_time_area_text','department_name','level'])->find();
                if($fence_info){
                    //$fence_info['department_name']=Db::table('departments')->where('id',$fence_info['departments'])->value('department_name');
                    $fence_list[]=$fence_info;
                }
                
            }
        }

        return CatchResponse::success($fence_list);
    }
 
    /**
     * 查询设备轨迹
     * @time 2022年09月18日 10:15
     * 
     */
    public function queryDeviceRoute(Request $request) : \think\Response
    {
      
        $params=$request->param();

        $device_number = $params['device_number'];
        $route_type = $params['route_type'];
 
        if(isset($params['filter_track']) && $params['filter_track']=='true'){
            $this->filterTrack = true;
        }else{
            $this->filterTrack = false;
        }
        if (!$device_number) {
            return CatchResponse::fail("获取设备号失败");
        }
        $start_time = strtotime(date('Y-m-d').'00:00:00');
        $end_time = strtotime(date('Y-m-d').'23:59:59');
        if(isset($params['time_range']) && $params['time_range'] != ''){
            $start_time=strtotime($params['time_range'][0]);
            $end_time=strtotime($params['time_range'][1]);
        }
        $options=array(
            'device_number'=>$device_number,
            'start_time'=>$start_time,
            'end_time'=>$end_time,
            'table_name'=>'wxt_route_gps'
        );
        $res = queryTableStoreGpsRoute($options);
        $rows=$res['data'];
        

        if(Env::get('map.wifiwayz')){
            $rows = $this->analyzeRouteInfoWayzBatch($rows,$route_type);
            $rows=bmapFilterOutliers($rows);
        }else{
            $rows = $this->analyzeRouteInfoGaodeBatch($rows,$route_type);
        }

        return CatchResponse::success($rows);
        
    }
    /**
     * 批量解析含wifi信号Gps  wayz
     */
    public function analyzeRouteInfoWayzBatch($list,$route_type='All'){

        if(!$list){
            return array();
        }
        $macLength=0;
        foreach($list as $item){
            if(isset($item['WifiMacs']) && $item['WifiMacs'] !=''){
                $macLength++;
            }
        }

        $arr= [];
        
        $preTime=0;
        $filter=0;//默认不过滤

        $wifiPreTime=0;
        $wifiFilter=0;//默认不过滤
        $count=count($list);
        if(!$this->filterTrack){
            
            if($count>1000){
                $filter=60;
            }elseif($count>500){
                $filter=30;
            }
            if($macLength>2000){
                $wifiFilter=600;
            }elseif($macLength>1000){
                $wifiFilter=300;
            }elseif($macLength>500){
                $wifiFilter=120;
            }else{
                $wifiFilter=30;
            }
        }
        $batch_wifi=array();
        foreach($list  as $k=> $v){
                if($filter>0){//gps点过多时间也过滤部分
                    $interval=abs($v['Timestamp']-$preTime);
                    if( $interval < $filter){
                        continue;
                    }
                }
                if(isset($v['WifiMacs']) && $v['WifiMacs'] !=''){
                    $wifiInterval=abs($v['Timestamp']-$wifiPreTime);
                    if( $wifiInterval < $wifiFilter){
                        continue;
                    }
                    if($route_type=='GPS'){
                        continue;
                    }
                    $batch_wifi[$k]=$v;
                    if(count($batch_wifi)==20 || $k==($count-1)){
                        $wifiRes =requestWifiLBS_wayz_batch($batch_wifi);
                        foreach($wifiRes as $key=>$value){
                            if($value['success']){
                                if($value['data']['lon'] == '' || is_null($value['data']['lat'])){
                                    continue;
                                }
                                $list[$key]['Latitude'] = $value['data']['lat'];
                                $list[$key]['Longitude'] = $value['data']['lon'];
                                $list[$key]['SignalType'] = 'WiFi';
                                // $list[$key]['WifiAddress'] = $value['data']['address'];
                                if(isset($value['data']['address'])){
                                    $list[$key]['WifiAddress'] = $value['data']['address'];
                                }
                                $arr[$key]=$list[$key];
                                //array_push($arr, $list[$k]);
                            }else{
                                continue;
                            }
                            
                        }
                        $batch_wifi=array();
                    }
                
                    $wifiPreTime=$v['Timestamp'];
                }else{
                    if($route_type=='WiFi'){
                        continue;
                    }
                    if($v['Altitude']=='0.003' || $v['Altitude']=='3'){
                        continue;
                    }
                    if( !isset($v['Latitude']) ||  !isset($v['Longitude']) || $v['Latitude']<0.065 || $v['Longitude']<0.065){
                        continue;
                    }
                    $list[$k]['SignalType'] = 'GPS';
                    $arr[$k]=$list[$k];
                    //array_push($arr, $list[$k]);
                }
                $preTime=$v['Timestamp'];
        }
        //防止最后剩余wifi点未更新

        if($batch_wifi){
            $wifiRes =requestWifiLBS_wayz_batch($batch_wifi);
            foreach($wifiRes as $key=>$value){
                    if($value['success']){
                        if($value['data']['lon'] == '' || is_null($value['data']['lat'])){
                            continue;
                        }
                        $list[$key]['Latitude'] = $value['data']['lat'];
                        $list[$key]['Longitude'] = $value['data']['lon'];
                        $list[$key]['SignalType'] = 'WiFi';
                        // $list[$key]['WifiAddress'] = $value['data']['address'];
                        if(isset($value['data']['address'])){
                            $list[$key]['WifiAddress'] = $value['data']['address'];
                        }
                        $arr[$key]=$list[$key];
                        //array_push($arr, $list[$k]);
                    }else{
                        continue;
                    }
                            
            }
            $batch_wifi=array();
        }
        ksort($arr);
        $response_data=[];
        foreach($arr as $item){
            array_push($response_data, $item);
        }
        return $response_data;
    }
    /**
     * 批量解析含wifi信号Gps  高德
     */
    public function analyzeRouteInfoGaodeBatch($list,$route_type='All'){

        if(!$list){
            return array();
        }
        $macLength=0;
        foreach($list as $item){
            if(isset($item['WifiMacs']) && $item['WifiMacs'] !=''){
                $macLength++;
            }
        }

        $arr= [];
        
        $preTime=0;
        $filter=0;//默认不过滤

        $wifiPreTime=0;
        $wifiFilter=0;//默认不过滤
        $count=count($list);
        if(!$this->filterTrack){
            
            if($count>1000){
                $filter=60;
            }elseif($count>500){
                $filter=30;
            }
            if($macLength>2000){
                $wifiFilter=600;
            }elseif($macLength>1000){
                $wifiFilter=300;
            }elseif($macLength>500){
                $wifiFilter=120;
            }else{
                $wifiFilter=30;
            }
        }
        $batch_wifi=array();
        foreach($list  as $k=> $v){
                if($filter>0){//gps点过多时间也过滤部分
                    $interval=abs($v['Timestamp']-$preTime);
                    if( $interval < $filter){
                        continue;
                    }
                }
                if(isset($v['WifiMacs']) && $v['WifiMacs'] !=''){
                    $wifiInterval=abs($v['Timestamp']-$wifiPreTime);
                    if( $wifiInterval < $wifiFilter){
                        continue;
                    }
                    if($route_type=='GPS'){
                        continue;
                    }
                    $batch_wifi[$k]=$v;
                    if(count($batch_wifi)==8|| $k==($count-1)){
                        $wifiRes =requestWifiLBS_gaode_batch($batch_wifi);
                        foreach($wifiRes as $key=>$value){
                            if($value['success']){
                                if($value['data']['lon'] == '' || is_null($value['data']['lat'])){
                                    continue;
                                }
                                $list[$key]['Latitude'] = $value['data']['lat'];
                                $list[$key]['Longitude'] = $value['data']['lon'];
                                $list[$key]['SignalType'] = 'WiFi';
                                $list[$key]['WifiAddress'] = $value['data']['address'];
                                $arr[$key]=$list[$key];
                                //array_push($arr, $list[$k]);
                            }else{
                                continue;
                            }
                            
                        }
                        $batch_wifi=array();
                    }
                
                    $wifiPreTime=$v['Timestamp'];
                }else{
                    if($route_type=='WiFi'){
                        continue;
                    }
                    if($v['Altitude']=='0.003' || $v['Altitude']=='3'){
                        continue;
                    }
                    if( !isset($v['Latitude']) ||  !isset($v['Longitude']) || $v['Latitude']<0.065 || $v['Longitude']<0.065){
                        continue;
                    }
                    $list[$k]['SignalType'] = 'GPS';
                    $arr[$k]=$list[$k];
                    //array_push($arr, $list[$k]);
                }
                $preTime=$v['Timestamp'];
        }
        if($batch_wifi){
            $wifiRes =requestWifiLBS_gaode_batch($batch_wifi);
            foreach($wifiRes as $key=>$value){
                if($value['success']){
                    if($value['data']['lon'] == '' || is_null($value['data']['lat'])){
                        continue;
                    }
                    $list[$key]['Latitude'] = $value['data']['lat'];
                    $list[$key]['Longitude'] = $value['data']['lon'];
                    $list[$key]['SignalType'] = 'WiFi';
                    $list[$key]['WifiAddress'] = $value['data']['address'];
                    $arr[$key]=$list[$key];
                    //array_push($arr, $list[$k]);
                }else{
                    continue;
                }
                
            }
            $batch_wifi=array();
        }
        $response_data=[];
        ksort($arr);
        foreach($arr as $key=>$item){
            array_push($response_data, $item);
        }
        return $response_data;
    }
    /**
     * 查询设备轨迹
     * @time 2022年09月18日 10:15
     * 
     */
    public function queryDeviceRouteOracle(Request $request) : \think\Response
    {
      
        $params=$request->param();

        $device_number = trim($params['device_number']);
       
        if (!$device_number) {
            return CatchResponse::fail("获取设备号失败");
        }
        $start_time = date('Y-m-d 00:00:00',time());
        $end_time = date('Y-m-d 23:59:59',time());

        if(isset($params['time_range']) && $params['time_range'] != ''){
            $start_time=date('Y-m-d H:i:s',strtotime($params['time_range'][0]));
            $end_time=date('Y-m-d H:i:s',strtotime($params['time_range'][1]));
        }


        $host=    Env::get('oracle.hostname',  '127.0.0.1');
        $port=    Env::get('oracle.hostport',  '1521');
        $instance_name=    Env::get('oracle.instance',  'ORCL');
        $username=    Env::get('oracle.username',  'root');
        $password=    Env::get('oracle.password',  'root');
        $conn = oci_connect($username, $password, $host.':'.$port.'/'.$instance_name,'AL32UTF8');

        if (!$conn) {
            $e = oci_error();
            trigger_error(htmlentities($e['message'], ENT_QUOTES), E_USER_ERROR);
        }

        $sql='SELECT * FROM (SELECT a.RF_ID,a.RF_FLAGID,to_char(a.RF_DATE,\'yyyy-mm-dd hh24:mi:ss\') RF_DATE,ROWNUM rn,b.DEVICE_NAME,b.DEVICE_CODE,b.LOGIN_NAME,c.GPS_X,c.GPS_Y FROM DSSC2.W_DW_RF_RECORD a join DSSC2.ADM_DEV b ON a.RF_ID=b.LOGIN_NAME join DSSC2.ADM_DEV_RFID_CHN c ON b.DEVICE_CODE=c.DEVICE_CODE WHERE  RF_FLAGID=\''.$device_number.'\' AND RF_DATE >= to_date(\''. $start_time.'\', \'yyyy-mm-dd hh24:mi:ss\') AND RF_DATE <= to_date(\''.$end_time.'\', \'yyyy-mm-dd hh24:mi:ss\')   ORDER BY RF_DATE DESC)   WHERE  ROWNUM <= 5000';
        // // 查询并渲染
        $stid = oci_parse($conn, $sql);
         // var_dump($stid);
        $r = oci_execute($stid);
        // var_dump($r);
        $rows=[];
        while($row = oci_fetch_array($stid, OCI_ASSOC+OCI_RETURN_NULLS)) {
            // $wgsLoc = \algorithm\Geometry::wgsTOgcj($row['GPS_Y'],$row['GPS_X']);
            $wgsLoc = \algorithm\Geometry::gcj02ToWgs84((float)$row['GPS_Y'],(float)$row['GPS_X']);
            // $lngLat = \algorithm\Geometry::convertGcj02ToBd09($wgsLoc['lat'],$wgsLoc['lng']);
            $row['GPS_X']=$wgsLoc['lng'];
            $row['GPS_Y']=$wgsLoc['lat'];
            $rows[]=$row;
        }
        // var_dump($rows);
        oci_free_statement($stid);
        oci_close($conn);
        // $conn = new PDO("oci:dbname=//".$host.":".$port."/".$instance_name,$username,$password);// PDO方式
        // $conn->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

        // // $sql='SELECT * FROM (SELECT a.*, ROWNUM rn FROM DSS2.W_DW_RF_RECORD a WHERE RF_FLAGID=\''.$device_number.'\' AND RF_DATE >= to_date(\''. $start_time.'\', \'yyyy-mm-dd hh24:mi:ss\') AND RF_DATE <= to_date(\''.$end_time.'\', \'yyyy-mm-dd hh24:mi:ss\')   ORDER BY RF_DATE DESC)   WHERE  ROWNUM <= 5000';
        // $sql='SELECT * FROM (SELECT a.RF_ID,a.RF_FLAGID,to_char(a.RF_DATE,\'yyyy-mm-dd hh24:mi:ss\') RF_DATE,ROWNUM rn,b.DEVICE_NAME,b.DEVICE_CODE,b.LOGIN_NAME,c.GPS_X,c.GPS_Y FROM DSSC2.W_DW_RF_RECORD a join DSSC2.ADM_DEV b ON a.RF_ID=b.LOGIN_NAME join DSSC2.ADM_DEV_RFID_CHN c ON b.DEVICE_CODE=c.DEVICE_CODE WHERE  RF_FLAGID=\''.$device_number.'\' AND RF_DATE >= to_date(\''. $start_time.'\', \'yyyy-mm-dd hh24:mi:ss\') AND RF_DATE <= to_date(\''.$end_time.'\', \'yyyy-mm-dd hh24:mi:ss\')   ORDER BY RF_DATE DESC)   WHERE  ROWNUM <= 5000';
        // // var_dump($sql);
        // $res = $conn -> query($sql);
        
        // $rows = $res -> fetchAll(PDO::FETCH_ASSOC);
        // foreach($rows as &$val){
        //     $val['DEVICE_NAME'] = mb_convert_encoding($val['DEVICE_NAME'], 'UTF-8', 'GBK');
        //     // $val['RF_DATE'] = mb_convert_encoding($val['RF_DATE'], 'UTF-8', 'GBK');
        //     $wgsLoc = \algorithm\Geometry::wgsTOgcj($val['GPS_Y'],$val['GPS_X']);
        //     $lngLat = \algorithm\Geometry::convertGcj02ToBd09($wgsLoc['lat'],$wgsLoc['lng']);
        //     $val['GPS_X']=$lngLat['lng'];
        //     $val['GPS_Y']=$lngLat['lat'];
        //     // $val['timeInt']=strtotime($val['RF_DATE']);
        // }
        // if ($conn){
        //     $conn = null;
        // }
        // $rows=[];
        return CatchResponse::success($rows);
        
    }
    /**
     * 查询设备轨迹
     * @time 2022年09月18日 10:15
     * 
     */
    public function queryDeviceRouteList(Request $request)
    {
      
        $params=$request->param();
        if($params['route_type']!='1'){
            $response=[
                'code'=>10000,
                'message'=>'查询成功',
                'count'=>0,
                'data'=>[],
                'current'=>isset($params['page'])?(int)$params['page']:1,
                'limit'=>isset($params['limit'])?(int)$params['limit']:10,
            ];
            return $response; 
        }
        $cond=[];
        // $cond=[
        //     '_string'=>'a.RF_FLAGID = s.RFID_SN AND o.rfid_id = s.id',
        // ];
        
        if($params['name']){
            $mac=Db::table('stations')->whereLike('name',$params['name'])->value('mac');
            $cond['RF_ID']=['=',$mac];
        }
        if($params['rfid_sn']){
            $cond['RF_FLAGID']=['=',$params['rfid_sn']];
        }
        $conn=getOracleConnect();
        if($params['plate_no']){
            $rfid_sn='';
            $vehicle_sql='SELECT s.RFID_SN FROM DSSC3.W_DW_NON_MOTOR o,DSSC3.W_DW_RFID_TAGS s WHERE o.PLATE_NO =\''.$params['plate_no'].'\'  AND o.rfid_id = s.id  ';
            $vtid = oci_parse($conn, $vehicle_sql);
            oci_define_by_name($vtid, 'RFID_SN', $rfid_sn);
            oci_execute($vtid);
            oci_fetch($vtid);
            $cond['RF_FLAGID']=['=',$rfid_sn];
        }
        $start_time = date('Y-m-d 00:00:00',time());
        $end_time = date('Y-m-d 23:59:59',time());
        if(isset($params['timeRange']) && $params['timeRange'] != ''){
            $start_time=date('Y-m-d H:i:s',strtotime($params['timeRange'][0]));
            $end_time=date('Y-m-d H:i:s',strtotime($params['timeRange'][1]));
            $cond['a.RF_DATE']=['timeRange',$start_time,$end_time];
        }
        // $count=queryOracleCount('DSSC2.W_DW_RF_RECORD a',$cond);
        
        $cond['page']=isset($params['page'])?$params['page']:1;
        $cond['limit']=isset($params['limit'])?$params['limit']:10;
        $rows=[];
        $rows=queryOracleSelect('(SELECT * FROM DSSC2.W_DW_RF_RECORD  ORDER BY ID DESC) a',$cond,'a.RF_FLAGID,a.RF_STAT,a.RF_ID,to_char(a.RF_DATE,\'yyyy-mm-dd hh24:mi:ss\') RF_DATE');
        
        foreach($rows as &$val){
            $vehicle_sql='SELECT o.PLATE_NO FROM DSSC3.W_DW_NON_MOTOR o,DSSC3.W_DW_RFID_TAGS s WHERE s.RFID_SN =\''.$val['RF_FLAGID'].'\'  AND o.rfid_id = s.id  ';
            $plate_no='';
            $stid = oci_parse($conn, $vehicle_sql);
            oci_define_by_name($stid, 'PLATE_NO', $plate_no);
            oci_execute($stid);
            oci_fetch($stid);
            oci_free_statement($stid);
            $val['PLATE_NO']=$plate_no;
            $val['ADDRESS']=Db::table('stations')->where('mac',$val['RF_ID'])->cache(120)->value('name');
            //状态: 0- 未知,1 - 进入,2 - 离开
            if($val['RF_STAT']==1){
                $val['RF_STAT_TEXT']='进入';
            }elseif($val['RF_STAT']==2){
                $val['RF_STAT_TEXT']='离开';
            }else{
                $val['RF_STAT_TEXT']='未知';
            }
        }
        
        oci_close($conn);
        $response=[
            'code'=>10000,
            'message'=>'查询成功',
            // 'count'=>$count,
            'data'=>$rows,
            'current'=>isset($params['page'])?(int)$params['page']:1,
            'limit'=>isset($params['limit'])?(int)$params['limit']:10,
        ];
        return $response;
        
    }
    /**
     * 统计条数
     * @time 2022年09月18日 10:15
     * 
     */
    public function queryDeviceRouteCount(Request $request)
    {
      
        $params=$request->param();
        if($params['route_type']!='1'){
            $response=[
                'code'=>10000,
                'message'=>'查询成功',
                'count'=>0,
            ];
            return $response; 
        }
        $cond=[];
        // $cond=[
        //     '_string'=>'a.RF_FLAGID = s.RFID_SN AND o.rfid_id = s.id',
        // ];
        
        if($params['name']){
            $mac=Db::table('stations')->whereLike('name',$params['name'])->value('mac');
            $cond['RF_ID']=['=',$mac];
        }
        if($params['rfid_sn']){
            $cond['RF_FLAGID']=['=',$params['rfid_sn']];
        }
       
        if($params['plate_no']){
            $conn=getOracleConnect();
            $rfid_sn='';
            $vehicle_sql='SELECT s.RFID_SN FROM DSSC3.W_DW_NON_MOTOR o,DSSC3.W_DW_RFID_TAGS s WHERE o.PLATE_NO =\''.$params['plate_no'].'\'  AND o.rfid_id = s.id  ';
            $vtid = oci_parse($conn, $vehicle_sql);
            oci_define_by_name($vtid, 'RFID_SN', $rfid_sn);
            oci_execute($vtid);
            oci_fetch($vtid);
            $cond['RF_FLAGID']=['=',$rfid_sn];
        }
        $start_time = date('Y-m-d 00:00:00',time());
        $end_time = date('Y-m-d 23:59:59',time());
        if(isset($params['timeRange']) && $params['timeRange'] != ''){
            $start_time=date('Y-m-d H:i:s',strtotime($params['timeRange'][0]));
            $end_time=date('Y-m-d H:i:s',strtotime($params['timeRange'][1]));
            $cond['a.RF_DATE']=['timeRange',$start_time,$end_time];
        }
        $count=queryOracleCount('DSSC2.W_DW_RF_RECORD a',$cond);

        $response=[
            'code'=>10000,
            'message'=>'查询成功',
            'count'=>$count,
           
        ];
        return $response;
        
    }
    /**
     * 统计条数
     * @time 2022年09月18日 10:15
     * 
     */
    public function queryCountSearchDevice(Request $request)
    {
      
        $params=$request->param();
        $cond=[];
        $keywords=$params['keywords'];
        if(!$keywords){
            return [
                'code'=>10000,
                'message'=>'查询成功',
                'station'=>0,
                'vehicle'=>0,
            ];
  
        }
        $station_count=Db::table('stations')->where('name|mac', 'like', '%'.$keywords.'%')->count();
       

        $conn=getOracleConnect();
        $COUNT=0;

        $vehicle_sql='SELECT count(*) as COUNT FROM DSSC3.W_DW_NON_MOTOR o,DSSC3.W_DW_RFID_TAGS s WHERE (s.RFID_SN like \'%'.$keywords.'%\' OR  o.PLATE_NO like \'%'.$keywords.'%\') AND o.rfid_id = s.id';
        $vtid = oci_parse($conn, $vehicle_sql);
        oci_define_by_name($vtid, 'COUNT', $COUNT);
        oci_execute($vtid);
        oci_fetch($vtid);
        oci_free_statement($vtid);
        oci_close($conn);
        

        $response=[
            'code'=>10000,
            'message'=>'查询成功',
            'station'=>$station_count,
            'vehicle'=>$COUNT,
           
        ];
        
        return $response;
        
    }
}